#include <QRandomGenerator>
#include <QCommandLineParser>
#include <QCoreApplication>
#include <QFile>
#include <QFileInfo>

#include <cmath>
#include <thread>
#include <ranges>

int main(int argc, char *argv[]) {
    QCoreApplication app(argc, argv);
    QCoreApplication::setApplicationName("Multi purpose mergesort application");
    QCoreApplication::setApplicationVersion("1.0.42");
    QTextStream print(stdout);


    QCommandLineParser parser;
    parser.setApplicationDescription("Used to run either sequential or parallel mergesort on a texfile containing ascii encoded int32s");
    parser.addHelpOption();
    parser.addVersionOption();

    QCommandLineOption sequential("S", "Run sequential sort on dataset");
    QCommandLineOption parallel("P", "Run parallel sort on dataset");
    QCommandLineOption nthreads(QStringList() << "d" << "depth", "Recursion depth of parallel part", "nthreads");
    QCommandLineOption output(QStringList() << "o", "output", "File to write the sorted dataset to");

    parser.addOption(sequential);
    parser.addOption(parallel);
    parser.addOption(nthreads);

    parser.addPositionalArgument("dataset", "Filename where to load the data from");

    parser.process(app);

    const int threads = std::thread::hardware_concurrency();
    int max_depth = std::sqrt(threads);

    print << "Hardware concurrency of " << threads << " detected" << Qt::endl;

    if(parser.isSet(nthreads)) {
        bool ok;
        max_depth = parser.value(nthreads).toInt(&ok);
        if(!ok) {
            parser.showHelp(-1);
        }
        print << "Overwriting maximum parallelized recursion depth with " << max_depth << Qt::endl;
    } else {
        print << "Assuming default parallelized recursion depth via sqrt(nthreads) of " << max_depth << Qt::endl;
    }



    app.exit(0);
    return 0;
}